@@ -23,6 +23,9 @@
 require 'Tioga/FigureConstants.rb'
 require 'Tioga/Utils'
 
+require "tmpdir"
+require "fileutils"
+
 module Tioga
 class FigureMaker
     
@@ -38,7 +41,7 @@ class FigureMaker
 
     # This URL will contain tioga-(...) when it is exported from the
     # SVN repository. This is where we'll look for version information.
-    SVN_URL = '$HeadURL: svn+ssh://rubyforge.org/var/svn/tioga/tags/tioga/Tioga%201.17/lib/Tioga/FigMkr.rb $'
+    SVN_URL = '$HeadURL: svn+ssh://rubyforge.org/var/svn/tioga/tags/tioga/Tioga%201.18/lib/Tioga/FigMkr.rb $'
 
     TIOGA_VERSION = if SVN_URL =~ /tags\/tioga\/Tioga%20([^\/]+)/
                        $1
@@ -165,6 +168,11 @@ class FigureMaker
     attr_accessor :measures_info
 
 
+    # Maximum number of pdflatex calls. If nil, then unlimited, but
+    # that could be a very bad idea.
+    attr_accessor :max_nested_pdflatex_calls
+    attr_reader   :current_pdflatex_call
+
 
     # If we want to use #legend_bounding_box. It is off by default
     # as it causes a systematic second run of pdflatex.
@@ -317,6 +325,9 @@ class FigureMaker
         # We don't measure legends by default.
         @measure_legends = false
 
+        # Max number of pdflatex runs
+        @max_nested_pdflatex_calls = 3
+
         # By default, we use Bill's algorithm for major ticks positions
         self.vincent_or_bill = false
         self.log_minor_ticks_limit = 10
@@ -1651,13 +1662,97 @@ class FigureMaker
     
     
     @@keys_for_show_image = FigureMaker.make_name_lookup_hash([
-        'll', 'lr', 'ul', 'w', 'width', 'height', 'h',
-        'opacity_mask', 'stencil_mask', 
+        'll', 'lr', 'ul', 'w', 'width', 'height', 'h', 'ref',
+        'opacity_mask', 'stencil_mask', 'bits', 'pdf_filters',
         'jpg', 'JPG', 'jpeg', 'JPEG', 'interpolate', 'data', 'value_mask',
         'color_space', 'color_map', 'colormap'])
     def show_image(dict)
         internal_show_image(dict, false)
     end
+
+    # Only register an image
+    def register_image(dict)
+        internal_show_image(dict, true)
+    end
+
+    def load_png(file)
+      if File.readable?(file)
+        Dir::mktmpdir do |dir|
+          FileUtils::cp(file, "#{dir}/image.png")
+          Dir::chdir(dir) do
+            open("read.tex", "w") do |f|
+              f.puts <<'EOD'
+\documentclass{article}
+\usepackage{graphicx}
+\pdfcompresslevel=0
+\pdfobjcompresslevel=0
+\begin{document}
+\includegraphics[width=1cm]{image}
+\end{document}
+EOD
+            end
+            pdflatex = "pdflatex"
+            syscmd = "#{pdflatex} -interaction nonstopmode read.tex"
+            IO::popen(syscmd, "r+") do |f|
+              f.close_write      
+              for line in f
+                # Shall we do something with those ?
+              end
+            end
+            
+            # Now, we open the PDF file and have fun with it.
+            open("read.pdf", 'rb') do |f|
+              
+              ret = nil
+              cur = nil
+              len = nil
+              while l = f.gets("\n")
+                if ! ret && l =~ /\/Subtype\s*\/Image\s*/
+                  ret = {}
+                  cur = ret
+                elsif cur && l=~ /\/Width\s*(\d+)\s*/
+                  cur['width'] = $1.to_i
+                elsif cur && l=~ /\/Height\s*(\d+)\s*/
+                  cur['height'] = $1.to_i
+                elsif cur && l =~ /\/Length\s*(\d+)\s*/
+                  len = $1.to_i
+                elsif cur && l =~ /\/(Filter|DecodePar)/ && (!cur['data'])
+                  cur['pdf_filters'] ||= ''
+                  cur['pdf_filters'] << l
+                elsif cur && l =~ /^\s*\/BitsPerComponent\s*(\d+)\s*/
+                  cur['bits'] = $1.to_i
+                elsif cur && l =~ /\/ColorSpace\s*(\S+)\s*/
+                  cs = $1
+                  if cs == "/DeviceGray"
+                    cs = 'gray'
+                  elsif cs == "/DeviceRGB"
+                    cs = "rgb"
+                  else
+                    raise "Unsupported colorspace for file #{file}: '#{cs}'"
+                  end
+                  cur['color_space'] = cs
+                elsif cur && l =~ /\/SMask\s*(\d+)\s*/
+                  cur['opacity_mask'] = {}
+                elsif cur && len && l =~ /stream/
+                  cur['data'] = f.read(len)
+                  if cur['opacity_mask']
+                    cur = cur['opacity_mask']
+                    len = nil
+                  else
+                    return ret
+                  end
+                end
+              end
+              return ret
+            end
+            
+          end
+        end
+      else
+        raise "File #{file} not readable"
+      end
+    end
+
     
     @@keys_for_create_colormap = FigureMaker.make_name_lookup_hash(['length', 'points', 'Rs', 'Gs', 'Bs', 'Hs', 'Ls', 'Ss'])
     def create_colormap(dict)
@@ -1944,19 +2039,20 @@ class FigureMaker
         elsif loc == BOTTOM
             shift = self.text_shift_on_bottom if shift == nil
         else
+            angle_add = 0
             if (loc == AT_X_ORIGIN)
                 shift = self.text_shift_from_x_origin if shift == nil
-                xloc = shift*self.char_height_dx
+                xloc = shift*self.default_text_height_dx
                 yloc = convert_frame_to_figure_y(position)
-                return
+                angle_add = 90
             elsif (loc == AT_Y_ORIGIN)
                 shift = self.text_shift_from_y_origin if shift == nil
-                yloc = shift*self.char_height_dy
+                yloc = shift*self.default_text_height_dx
                 xloc = convert_frame_to_figure_x(position)
             else
                 raise "Sorry: 'loc' must be LEFT, RIGHT, TOP, BOTTOM, AT_X_ORIGIN, or AT_Y_ORIGIN for show_text"
             end
-            show_rotated_label(text, xloc, yloc, scale, angle, just, align,
+            show_rotated_label(text, xloc, yloc, scale, angle + angle_add, just, align,
                                dict['measure'])
             return
         end
@@ -2148,17 +2244,25 @@ class FigureMaker
         num = get_num_for_pdf(num)
         result = start_making_pdf(num)
         return unless result
+
+        @current_pdflatex_call ||= 0
         begin
         @figure_pdfs[num] = finish_making_pdf(@figure_names[num])
         # If the keys have changed, we run that again.
         rescue Exception => e
           p e, e.backtrace
         end
-          
+
         if @measures.keys != old_measure_keys
-          # we dont need to pass &cmd since it has now been defined
-          make_pdf(num)
+          @current_pdflatex_call += 1
+
+          # We limit
+          if (!@max_nested_pdflatex_calls || (@current_pdflatex_call < @max_nested_pdflatex_calls))
+            # we dont need to pass &cmd since it has now been defined
+            make_pdf(num)
+          end
         end
+        @current_pdflatex_call = nil
         return @figure_pdfs[num]
     end
     
@@ -2390,6 +2494,12 @@ EOE
         # p @fm_data
         private_save_measure(key, *val)
       end
+
+      # Delete all keys in @
+      extra = @measures_info.keys - @measures.keys
+      for e in extra
+        @measures_info.delete(e)
+      end
         
       result = $?
         if !result
@@ -2441,74 +2551,71 @@ EOE
         return false
     end
 
-    def internal_show_image(dict, is_mask)
-        check_dict(dict, @@keys_for_show_image, 'show_image')
-        ll = dict['ll']; lr = dict['lr']; ul = dict['ul']
-        w = alt_names(dict, 'w', 'width')
-        h = alt_names(dict, 'h', 'height')
-        opacity_mask = alt_names(dict, 'opacity_mask', 'stencil_mask')
-        if opacity_mask != nil
-            mask_xo_num = internal_show_image(opacity_mask, true)
-        else
-            mask_xo_num = 0
-        end
-        filename = alt_names(dict, 'jpg', 'JPG')
-        filename = dict['jpeg'] if filename == nil
-        filename = dict['JPEG'] if filename == nil
-        if filename != nil
-            return private_show_jpg(filename, w, h, [ll[0], ll[1], lr[0], lr[1], ul[0], ul[1]], mask_xo_num)
-        end
+    def internal_show_image(dict, register_only)
+      check_dict(dict, @@keys_for_show_image, 'show_image')
+      ll = dict['ll']; lr = dict['lr']; ul = dict['ul']
+      w = alt_names(dict, 'w', 'width')
+      h = alt_names(dict, 'h', 'height')
+      opacity_mask = alt_names(dict, 'opacity_mask', 'stencil_mask')
+      bits = dict['bits'] || 8
+      filters = dict['pdf_filters'] || nil
+      ref = dict['ref'] || nil
+      if opacity_mask != nil
+        om = opacity_mask.dup
+        mask_xo_num = om['ref'] || internal_show_image(om, true)
+      else
+        mask_xo_num = 0
+      end
+      filename = alt_names(dict, 'jpg', 'JPG')
+      filename = dict['jpeg'] if filename == nil
+      filename = dict['JPEG'] if filename == nil
+      if ref
+        # Nothing to register, just reuse the image
+      elsif filename != nil
+        ref =  private_register_jpg(filename, w, h, mask_xo_num)
+      else
         interpolate = get_if_given_else_default(dict, 'interpolate', true)
         data = dict['data']
         value_mask = dict['value_mask']
         color_space = alt_names(dict, 'color_space', 'colormap')
         color_space = dict['color_map'] if color_space == nil
-        if color_space == nil
-            raise "Sorry: must specify 'color_space' for the image"
-        end
-        if color_space == 'MONO' || color_space == 'mono'
-            raise "Sorry: monochrome image must not itself have a mask" unless mask_xo_num == 0
-            reversed = get_if_given_else_default(dict, 'reversed', false)
-            xo_obj = private_show_monochrome_image(ll[0], ll[1], lr[0], lr[1], ul[0], ul[1], 
-                            interpolate, reversed, w, h, data, ((is_mask)? -1 : 0))
-            return is_mask ? xo_obj : self
-        end
-        if color_space == 'GRAY' || color_space == 'gray' || color_space == 'GREY' || color_space == 'grey'
-            if is_mask
-                raise("Sorry: mask must not itself have a mask") unless mask_xo_num == 0
-                mask_xo_num = -1
-            end
-            xo_obj = private_show_grayscale_image(ll[0], ll[1], lr[0], lr[1], ul[0], ul[1],
-                            interpolate, w, h, data, mask_xo_num)
-            return is_mask ? xo_obj : self
-        end
-        if is_mask
-            raise "Sorry: mask image must have 'color_space' set to 'gray' or 'mono'"
-        end
-        if color_space == 'RGB' || color_space == 'rgb'
-            private_show_rgb_image(ll[0], ll[1], lr[0], lr[1], ul[0], ul[1], interpolate, w, h, data, mask_xo_num)
-            return self
-        end
-        if color_space == 'HLS' || color_space == 'hls'
-            private_show_hls_image(ll[0], ll[1], lr[0], lr[1], ul[0], ul[1], interpolate, w, h, data, mask_xo_num)
-            return self
-        end
-        if color_space == 'CMYK' || color_space == 'cmyk'
-            private_show_cmyk_image(ll[0], ll[1], lr[0], lr[1], ul[0], ul[1], interpolate, w, h, data, mask_xo_num)
-            return self
-        end
-        if value_mask == nil
+        case color_space
+        when nil
+          raise "Sorry: must specify 'color_space' for the image"
+        when 'MONO', 'mono'
+          raise "Sorry: monochrome image must not itself have a mask" unless mask_xo_num == 0
+          reversed = get_if_given_else_default(dict, 'reversed', false)
+          ref = private_register_monochrome_image(interpolate, reversed, w, h, data, ((register_only)? -1 : 0), filters)
+        when 'GRAY', 'gray', 'GREY', 'grey'
+          if register_only
+            raise("Sorry: mask must not itself have a mask") unless mask_xo_num == 0
+            mask_xo_num = -1
+          end
+          ref = private_register_grayscale_image(interpolate, w, h, data, mask_xo_num, bits, filters)
+        when 'RGB', 'rgb'
+          ref = private_register_rgb_image(interpolate, w, h, data, mask_xo_num, bits, filters)
+        when 'HLS', 'hls'
+          ref = private_register_hls_image(interpolate, w, h, data, mask_xo_num, bits, filters)
+        when 'CMYK', 'cmyk'
+          ref = private_register_cmyk_image(interpolate, w, h, data, mask_xo_num, bits, filters)
+        else 
+          if value_mask == nil
             value_mask_min = value_mask_max = 256
-        elsif value_mask.kind_of?Integer
+          elsif value_mask.kind_of?Integer
             value_mask_min = value_mask_max = value_mask
-        else
+          else
             value_mask_min = value_mask[0];
             value_mask_max = value_mask[1];
+          end
+          ref = private_register_image(interpolate, 
+                                   w, h, data, value_mask_min, value_mask_max, color_space[0], color_space[1], mask_xo_num, bits, filters)
         end
-        private_show_image(
-                    ll[0], ll[1], lr[0], lr[1], ul[0], ul[1], interpolate, 
-                    w, h, data, value_mask_min, value_mask_max, color_space[0], color_space[1], mask_xo_num)
-        return self
+      end
+      if ! register_only
+        private_show_image_from_ref(ref, 
+                                    ll[0], ll[1], lr[0], lr[1], ul[0], ul[1])
+      end
+      return ref
     end
     
     def report_error(er, msg)
